#!/usr/bin/perl
# $Id: init_subnets.pl,v 1.3 2004/01/27 17:47:42 bzizou Exp $ #
# 
# This script:
# - Creates the tables "subnets" and "subnetstat" that contains the subnets and
# associated schedules
# - Reads the file "default_subnets" from the DKS to init the tables
# - Reads the file "default_schedules" from the DKS to put the default schedules
# in the tables
#

use DBI;
use Sys::Syslog qw(:DEFAULT setlogsock);
use NetAddr::IP;

# Load the lib
do ("/usr/local/lib/slis-sys.inc.pl");

# Load the SLIS variables
load_config();

# This script depends on the gen_iplist.pl script
$GEN_IPLIST_CMD="/usr/local/sbin/gen_iplist.pl";

# Open syslog
init_syslog('setup.schedules');

# Load language file
load_language;

# SQL connection
$dsn=sql_init();
$dbh = DBI->connect($dsn,$ADMINUSER,$ADMINPASS);

# File opening
if (defined($ARGV[0]) && defined($ARGV[1])) { $subnets=$ARGV[0]; $schedules=$ARGV[1]; }
else {
  log_msg("Usage: setup.schedules <default_subnets_file> <default_schedules_file>");
  print "Usage: setup.schedules <default_subnets_file> <default_schedules_file>\n";
  exit 1;
}
if (!open (SUBNETS,"$subnets")) {
  log_msg("Unable to open file $subnets!");
  print "Unable to open file $subnets!\n";
  exit 1;
}
if (!open (SCHEDULES,"$schedules")) {
  log_msg("Unable to open file $schedules!");
  print "Unable to open file $schedules!\n";
  exit 1;
}


# TABLES CREATION
#################
$sth = $dbh->prepare("DROP table subnets");
$sth->execute;
$sth = $dbh->prepare("DROP table subnetstat");
$sth->execute;
$sth = $dbh->prepare("DROP sequence subnet_id");
$sth->execute;
$sth = $dbh->prepare("DROP sequence bar_id");
$sth->execute;
$sth = $dbh->prepare("CREATE SEQUENCE subnet_id
  start 1 increment 1 maxvalue 2147483647 minvalue 1 cache 1;     
");
$sth->execute;
$sth = $dbh->prepare("CREATE SEQUENCE bar_id 
  start 1 increment 1 maxvalue 2147483647 minvalue 1 cache 1;
");
$sth->execute;
$sth = $dbh->prepare(" 
  CREATE TABLE subnets (
   id int4 DEFAULT nextval('subnet_id'::text) NOT NULL,
   nom varchar(30) NOT NULL,
   network varchar(15) NOT NULL,
   netmask varchar(15) NOT NULL,
   status bool,
   type int2,
   PRIMARY KEY (id)
  );
");
$sth->execute;
log_err($sth,'create table subnets');
$sth = $dbh->prepare("CREATE INDEX id_subnets_key ON subnets (id);");
$sth->execute;
$sth = $dbh->prepare("CREATE INDEX nom_subnets_key ON subnets (nom);");
$sth->execute;
$sth = $dbh->prepare("CREATE INDEX status_subnets_key ON subnets (status);");
$sth->execute;
$sth = $dbh->prepare("CREATE INDEX type_subnets_key ON subnets (type);");
$sth->execute;
$sth = $dbh->prepare(" 
  CREATE TABLE subnetstat (
   id int4 DEFAULT nextval('bar_id'::text) NOT NULL,
   subid int4 NOT NULL,
   actif int4 NOT NULL,
   inactif int4 NOT NULL,
   days int4 NOT NULL,
   PRIMARY KEY (id)
  );
");
$sth->execute;
log_err($sth,'create table subnetstat');
$sth = $dbh->prepare("CREATE INDEX id_subnetstat_key ON subnetstat (id);");
$sth->execute;
$sth = $dbh->prepare("CREATE INDEX subid_subnetstat_key ON subnetstat (subid);");
$sth->execute;
$sth = $dbh->prepare("CREATE INDEX days_subnetstat_key ON subnetstat (days);");
$sth->execute;


# LOADING OF THE DEFAULT SCHEDULES
##################################
while (<SCHEDULES>) {
  chomp;
  if (not ($_ =~ /^\s*\#.*/) && not (($_ =~ /^\s*$/))) { # skips comments and empty lines
    ($day,$h_start,$h_stop)=split(/,/);
    if ("$h_start" ne "" && "$h_stop" ne "") {
      $start{"$day"}=$h_start * 60;
      $stop{"$day"}=$h_stop * 60;
    }
  }
}

# LOADING OF THE SUBNET FILE AND SQL INSERTS
############################################
$id=0;

# Fonction that tests and inserts a subnet into the SQL table
sub insert_subnet {
  if ($_[0] eq "ALL") {
    $ip=$NETWORK1; $mask=$NETMASK1;
  }
  else {
    ($ip,$mask)=split();
  }
  $space=new NetAddr::IP $ip, $mask;
  if (!$space) {
    log_msg("$ip/$mask is not a valid network!");
    print"$ip/$mask is not a valid network!\n";
  }
  else {
    $id++;
    if ($_[0] eq "ALL") { 
      if ($LANG==fr) { $name="TOUT";} else { $name = "ALL";}
    }
    else { $name = $id; }
    $sth = $dbh->prepare("INSERT INTO subnets
      (nom,network,netmask,status,type)
      values ('$name','$ip','$mask','t','0')
    ");
    $sth->execute;
    for ($d=1;$d<=7;$d++) {
      if ($start{"$d"} != 0 || $stop{"$d"} != 0) {
        $start=$start{"$d"};
        $stop=$stop{"$d"};
        $sth = $dbh->prepare("INSERT INTO subnetstat
           (subid,actif,inactif,days)
              values
           ('$id','$start','$stop','$d')
        ");
        $sth->execute;
      }
    }
  }
}

# Loop over the subnets file
while (<SUBNETS>) {
  chomp;
  if (not ($_ =~ /^\s*\#.*/) && not (($_ =~ /^\s*$/))) { # skips comments and empty lines

    # Auto mode, create subnets by using gen_iplist.pl
    if ($_ eq "auto" || $_ eq "AUTO") {
      @subnets=`$GEN_IPLIST_CMD -n -i $NETWORK1 -m $NETMASK1`;
      foreach (@subnets) {
        insert_subnet();
      }
      # The last subnet is the whole network
      insert_subnet("ALL");
      exit 0;
    }

    # Manual mode: insert the subnets given    
    else { insert_subnet(); }
  }
}
# The last subnet is the whole network
insert_subnet("ALL");


