#!/bin/bash
# $Id: restore-2.x.bash,v 3.7 2004/01/30 15:10:22 bzizou Exp $
# Restore script for SLIS 2.x backup


### VARIABLES ###

# Load the SLIS lib
if [ -f /usr/local/lib/slis-sys.inc.bash ]
then
  . /usr/local/lib/slis-sys.inc.bash
else
  logger -t "bash_slis" "ERROR: /usr/local/lib/slis-sys.inc.bash not found."
  echo "ERROR: /usr/local/lib/slis-sys.inc.bash not found."
  exit 1
fi

# Load the variables
load_config

# Get the variables from the PHP config script
export PGUSER=`grep "POSTGRESUSER=" /home/hadmin/html/config_pgsql.inc.php |cut -d\" -f2`
export PGPASSWORD=`grep "POSTGRESPASS=" /home/hadmin/html/config_pgsql.inc.php |cut -d\" -f2`
PGSQL_SERVER=`grep "PGSQL_SERVER=" /home/hadmin/html/config_pgsql.inc.php |cut -d\" -f2`
BASE_HADMIN=`grep "BASE_HADMIN=" /home/hadmin/html/config_pgsql.inc.php |cut -d\" -f2`

# Other variables
TMP_DIR="/home/hadmin/tmp/backup"
SQL_QUERY="psql -h $PGSQL_SERVER $BASE_HADMIN -tqc"



### Main Database restore ###

echo "Cleaning the database..."
# Clean the database
$SQL_QUERY "delete from users"
$SQL_QUERY "delete from classes"
$SQL_QUERY "delete from groups"
$SQL_QUERY "delete from subnets"
$SQL_QUERY "delete from subnetstat"
$SQL_QUERY "delete from alias"
$SQL_QUERY "delete from oldvalue"
$SQL_QUERY "delete from badlogin"
$SQL_QUERY "delete from password"
$SQL_QUERY "delete from security_filters"

# Restore the SQL dump
echo "Restoring the dump..."
perl -pi -e "s/\\\connect - postgres//" $TMP_DIR/home/hadmin/hadmin.sql
egrep -v "reltriggers|pg_trigger" $TMP_DIR/home/hadmin/hadmin.sql > $TMP_DIR/home/hadmin/hadmin.sql.wrk
export PGUSER=`grep "ADMINUSER=" /home/hadmin/html/config_pgsql.inc.php |cut -d\" -f2`
export PGPASSWORD=`grep "ADMINPASS=" /home/hadmin/html/config_pgsql.inc.php |cut -d\" -f2`
psql -tq -f $TMP_DIR/home/hadmin/hadmin.sql.wrk hadmin 2>&1 |egrep -v "already exists|duplicate|lost synchronization|NOTICE|multiple primary keys"

# Set the flags on
echo "Updating the flags..."
$SQL_QUERY "update users set nouveau='1',tovalid='1'"
$SQL_QUERY "update alias set nouveau='1',tovalid='1'"
$SQL_QUERY "update groups set nouveau='1',tovalid='1'"
$SQL_QUERY "update classes set nouveau='1',tovalid='1'"

# Initialize the passwords
declare -i VERSION
VERSION=`cat /etc/version_slis_num|cut -f1 -d.`
if [ $VERSION -gt 2 ]
then
  echo "Initializing the SQL password (must'n be empty)..."
  for id in `psql hadmin -tqc "select id from users"`
  do
    PASS=`/usr/local/sbin/pspg`
    psql hadmin -tqc "update users set passwd='$PASS' where id='$id'"
  done
fi

echo "Restoring DHCP dump..."
### DHCP database restore ###
export PGUSER=`grep "POSTGRESUSER=" /home/hadmin/html/config_pgsql.inc.php |cut -d\" -f2`
export PGPASSWORD=`grep "POSTGRESPASS=" /home/hadmin/html/config_pgsql.inc.php |cut -d\" -f2`
SQL_QUERY="psql -h $PGSQL_SERVER dhcp -tqc"
$SQL_QUERY "delete from materiel"
$SQL_QUERY "delete from reseau"
$SQL_QUERY "delete from constructeurs"
$SQL_QUERY "delete from arp"
psql -tq -f $TMP_DIR/home/hadmin/dhcp.sql dhcp 2>&1 |egrep -v "already exists|NOTICE|multiple primary keys"


echo "Restoring B2I dump..."
### B2i database restore (B2i is a little french application) ###
export PGUSER=`grep "SLISUSER=" /home/hadmin/html/config_pgsql.inc.php |cut -d\" -f2`
export PGPASSWORD=`grep "SLISPASS=" /home/hadmin/html/config_pgsql.inc.php |cut -d\" -f2`
psql template1 -tqc "drop database b2i"
psql template1 -tqc "create database b2i"
psql -tq b2i < $TMP_DIR/home/hadmin/b2i.sql


echo "Fixing the sequences..."
### Fix the tables (sequences problem)
/usr/local/sbin/fix_tables

### Synchro with the system
echo "Starting the validation script..."
/usr/local/sbin/valid
echo "Synchronization with ldap..."
/usr/local/sbin/synchro_ldap.pl

echo "Restoring the LDAP passwords..."
### LDAP passwords restore
cd $TMP_DIR/etc
PASS_LDAP=`cat /etc/openldap/secret`
if [ "`cat version_slis |cut -f1 -d.`" = "2" ]
then
  egrep -v '^root:|^bin:|^daemon:|^adm:|^lp:|^sync:|^shutdown:|^halt:|^mail:|^news:|^uucp:|^operator:|^games:|^gopher:|^ftp:|^nobody:|^xfs:|^named:|^assistance:|^postgres:|^hadmin:|^mailnull:|^ldap:|^pcap:|^squid:|^bzeznik:' shadow > shadow.wrk
  awk -F: -vU="$LDAP_USERS" '{print "dn: uid=" $1 "," U "\nuserpassword: {CRYPT}" $2 "\n"}' shadow.wrk >$TMP_DIR/home/hadmin/passwords.ldif 
  cat $TMP_DIR/home/hadmin/passwords.ldif |ldapmodify -c -D "$LDAP_ROOT_DN" -x -w $PASS_LDAP >/dev/null
fi
VERSION=`cat /etc/version_slis_num|cut -d. -f1`
if [ "$VERSION" != "2" ]
then
  sed "s/%BASE%/$LDAP_USERS/" $TMP_DIR/home/hadmin/passwords.ldif \
    | ldapmodify -c -D "$LDAP_ROOT_DN" -x -w $PASS_LDAP >/dev/null
fi

############
# Futur obsolete things...

cp -f $TMP_DIR/home/hadmin/cgi-bin/base/perso_* /home/hadmin/cgi-bin/base
cp -f $TMP_DIR/home/hadmin/cgi-bin/base/squidguard_databases /home/hadmin/cgi-bin/base
cp -f $TMP_DIR/home/hadmin/cgi-bin/base/wget-auto.slis /home/hadmin/cgi-bin/base
cp -f $TMP_DIR/home/hadmin/cgi-bin/base/wget.slis /home/hadmin/cgi-bin/base

# Reset good permissions
echo "Reseting permissions..."
/usr/local/sbin/permslis
echo "Done."
